{*******************************************************}
{                                                       }
{       Animated Menus                                  }
{       T_AM2000_DropShadowForm                         }
{                                                       }
{       Copyright (c) 1997-2001 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000dropshadow;

{$I am2000.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, am2000utils;


type
  // popup menu

  T_AM2000_DropShadowForm = class(TForm)
  private
    DesktopDC: HDC;
    FDepth, OldW, OldH, OldX, OldY: Integer;
    vbi, hbi: TBitmapInfo;
    vbmp, hbmp, Desktop, tempbmp: TBitmap;

    procedure SetDepth(Value: Integer);

  protected
    procedure Paint; override;

  public
    DirectionX, DirectionY: Boolean;

    property Depth: Integer
      read FDepth write SetDepth default 4;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Refresh;
    procedure GetDesktop(X, Y, MX, MY: Integer; CanReuse: Boolean);
    procedure ClearDesktop;

  end;


implementation

{ T_AM2000_DropShadowForm }

constructor T_AM2000_DropShadowForm.Create(AOwner: TComponent);
begin
  inherited CreateNew(AOwner {$IFDEF CBuilder1}, 0{$ENDIF});

  ControlStyle:= ControlStyle - [csCaptureMouse];
  BorderStyle:= bsNone;
  FormStyle:= fsStayOnTop;
  Position:= poDesigned;
  SetBounds(0, 0, 0, 0);

  vbmp:= TBitmap.Create;
  hbmp:= TBitmap.Create;
  Desktop:= TBitmap.Create;
  tempbmp:= TBitmap.Create;

  DesktopDC:= CreateDC('DISPLAY', nil, nil, nil);

  vbi.bmiHeader.biSize:= SizeOf(vbi.bmiHeader);
  vbi.bmiHeader.biWidth:= FDepth;
  vbi.bmiHeader.biPlanes:= 1;
  vbi.bmiHeader.biBitCount:= 24;
  vbi.bmiHeader.biCompression:= bi_RGB;
  vbi.bmiHeader.biSizeImage:= 0;

  hbi.bmiHeader.biSize:= SizeOf(hbi.bmiHeader);
  hbi.bmiHeader.biHeight:= FDepth;
  hbi.bmiHeader.biPlanes:= 1;
  hbi.bmiHeader.biBitCount:= 24;
  hbi.bmiHeader.biCompression:= bi_RGB;
  hbi.bmiHeader.biSizeImage:= 0;

  Depth:= 4;
end;

destructor T_AM2000_DropShadowForm.Destroy;
begin
  // free bits
  vbmp.Free;
  hbmp.Free;
  Desktop.Free;
  tempbmp.Free;
  DeleteDC(DesktopDC);

  inherited Destroy;
end;

procedure T_AM2000_DropShadowForm.Paint;
var
  bs, I, H, W, W1, X, Y, DI: Integer;
  bits: PByteArray;

  procedure CalculateVerticalDropShadow(const Index, d1, d2, d3, d4: Integer);
  begin
    bits^[Index]:=     d1 * (bits^[Index] div 9);
    bits^[Index +1]:=  d1 * (bits^[Index +1] div 9);
    bits^[Index +2]:=  d1 * (bits^[Index +2] div 9);
    bits^[Index +3]:=  d2 * (bits^[Index +3] div 9);
    bits^[Index +4]:=  d2 * (bits^[Index +4] div 9);
    bits^[Index +5]:=  d2 * (bits^[Index +5] div 9);
    bits^[Index +6]:=  d3 * (bits^[Index +6] div 9);
    bits^[Index +7]:=  d3 * (bits^[Index +7] div 9);
    bits^[Index +8]:=  d3 * (bits^[Index +8] div 9);
    bits^[Index +9]:=  d4 * (bits^[Index +9] div 9);
    bits^[Index +10]:= d4 * (bits^[Index +10] div 9);
    bits^[Index +11]:= d4 * (bits^[Index +11] div 9);
  end;

  procedure CalculateHorizontalDropShadow(const Index, d1: Integer);
  var
    J, J0, J1: Integer;
  begin
    J0:= Index * W1 * SizeOf(TRgbTriple) + DI * Index;
    J1:= (Index + 1) * W1 * SizeOf(TRgbTriple) + DI * Index -1;
    if J1 >= bs then J1:= bs -1;

    for J:= J0 + 12 to J1
    do bits^[J]:= d1 * (bits^[J] div 9);
  end;

begin
  if (csLoading in ComponentState)
  or (csDestroying in ComponentState)
  or (Desktop.Height < Height + FDepth)
  or (Desktop.Width < Width + FDepth)
  then Exit;

  H:= Height;
  W:= Width;
  W1:= W - FDepth;
  DI:= W1 mod 4;

  // calc direction
  if DirectionX
  then X:= FDepth
  else X:= Desktop.Width - W - FDepth;

  if DirectionY
  then Y:= FDepth
  else Y:= Desktop.Height - H - FDepth;

  if (OldW <> W)
  or (OldH <> H)
  then begin
    // vertical shadow
    vbmp.Height:= H;
    BitBlt(vbmp.Canvas.Handle, 0, 0, FDepth, H, Desktop.Canvas.Handle, W, Y, SrcCopy);

    // get dib bits
    bs:= (FDepth +1)* H * SizeOf(TRgbTriple);
    GetMem(bits, bs);

    vbi.bmiHeader.biHeight:= H;
    GetDIBits(vbmp.Canvas.Handle, vbmp.Handle, 0, H, bits, vbi, Dib_Rgb_Colors);

    for I:= 3 to H -4
    do CalculateVerticalDropShadow(I *12, 5, 6, 7, 8);

    // calc bottom
    CalculateVerticalDropShadow(0,  8, 8, 8, 8);
    CalculateVerticalDropShadow(12, 7, 7, 7, 8);
    CalculateVerticalDropShadow(24, 6, 6, 7, 8);

    // calc top
    CalculateVerticalDropShadow((H -1)*12, 8, 8, 8, 8);
    CalculateVerticalDropShadow((H -2)*12, 7, 7, 7, 8);
    CalculateVerticalDropShadow((H -3)*12, 6, 6, 7, 8);

    SetDIBits(vbmp.Canvas.Handle, vbmp.Handle, 0, H, bits, vbi, Dib_Rgb_Colors);
    FreeMem(bits);


    // horizontal
    hbmp.Width:= W1;
    BitBlt(hbmp.Canvas.Handle, 0, 0, W1, FDepth, Desktop.Canvas.Handle, X, H, SrcCopy);

    bs:= FDepth * W1 * SizeOf(TRgbTriple) + FDepth + 100;
    GetMem(bits, bs);

    hbi.bmiHeader.biWidth:= W1;
    GetDIBits(hbmp.Canvas.Handle, hbmp.Handle, 0, FDepth, bits, hbi, Dib_Rgb_Colors);

    for I:= 0 to FDepth -1
    do CalculateHorizontalDropShadow(I, 8 - I);

    // calc bottom
    I:= W1 * SizeOf(TRgbTriple) + DI;
    CalculateVerticalDropShadow(0, 8, 8, 8, 8);
    CalculateVerticalDropShadow(I, 8, 7, 7, 7);
    CalculateVerticalDropShadow(2*I, 8, 7, 6, 6);
    CalculateVerticalDropShadow(3*I, 8, 7, 6, 5);

    SetDIBits(hbmp.Canvas.Handle, hbmp.Handle, 0, FDepth, bits, hbi, Dib_Rgb_Colors);
    FreeMem(bits);
  end;

  // draw
  BitBlt(Canvas.Handle, W - FDepth, 0, FDepth, H, vbmp.Canvas.Handle, 0, 0, SrcCopy);
  BitBlt(Canvas.Handle, 0, H - FDepth, W1, FDepth, hbmp.Canvas.Handle, 0, 0, SrcCopy);

  OldW:= W;
  OldH:= H;
end;

procedure T_AM2000_DropShadowForm.Refresh;
var
  F: TForm;
begin
  if Owner is TForm
  then begin
    F:= TForm(Owner);
    SetBounds(F.Left + FDepth, F.Top + FDepth, F.Width, F.Height);
    Paint;
  end;
end;

procedure T_AM2000_DropShadowForm.GetDesktop(X, Y, MX, MY: Integer; CanReuse: Boolean);
begin
  if CanReuse
  then tempbmp.Assign(Desktop);

  // get desktop
  Desktop.Width:= MX + FDepth;
  Desktop.Height:= MY + FDepth;
  BitBlt(Desktop.Canvas.Handle, 0, 0, MX + FDepth, MY + FDepth, DesktopDC, X, Y, SrcCopy);

  if CanReuse
  then Desktop.Canvas.Draw(OldX - X, OldY - Y, tempbmp);

  OldX:= X;
  OldY:= Y;
end;

procedure T_AM2000_DropShadowForm.ClearDesktop;
begin
  OldW:= 0;
  OldH:= 0;
end;

procedure T_AM2000_DropShadowForm.SetDepth(Value: Integer);
begin
  FDepth:= Value;
  vbi.bmiHeader.biWidth:= Value;
  hbi.bmiHeader.biHeight:= Value;
  vbmp.Width:= FDepth;
  hbmp.Height:= FDepth;
end;

end.
